"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var constants_1 = require("./constants");
var is_array_buffer_1 = require("@aws-sdk/is-array-buffer");
/**
 * Parses well-formed HTTP request strings into HTTP request objects.
 *
 * This function was written to facilitate working with wire logs rather than to
 * be part of the critical path of an HTTP stack; it was not designed with
 * performance in mind.
 *
 * @see https://tools.ietf.org/html/rfc7230#section-3
 */
function parseRequest(serialized) {
    var _a = parseMessage(serialized), startLine = _a.startLine, headers = _a.headers, body = _a.body;
    if (startLine.match(/^\S+\s+\S+\s+\S+/)) {
        var _b = tslib_1.__read(startLine.split(/\s+/, 3), 3), method = _b[0], target = _b[1], protocolVersion = _b[2];
        var message = {
            method: method,
            headers: headers,
            body: body,
        };
        parseTargetIntoMessage(message, target);
        if (!message.hostname && headers['host']) {
            var matches = headers.host.match(/:(\d+)$/);
            if (matches) {
                message.hostname = headers.host.substr(0, matches.index);
                message.port = parseInt(matches[1], 10);
            }
            else {
                message.hostname = headers.host;
            }
        }
        return message;
    }
    throw new Error('Invalid request string');
}
exports.parseRequest = parseRequest;
/**
 * Parses well-formed HTTP response strings into HTTP response objects.
 *
 * This function was written to facilitate working with wire logs rather than to
 * be part of the critical path of an HTTP stack; it was not designed with
 * performance in mind.
 *
 * @see https://tools.ietf.org/html/rfc7230#section-3
 */
function parseResponse(serialized) {
    var _a = parseMessage(serialized), startLine = _a.startLine, headers = _a.headers, body = _a.body;
    if (startLine.match(/^HTTP\/.* [0-9]{3}( .*|$)/)) {
        var _b = tslib_1.__read(startLine.split(' ', 3), 3), protocol = _b[0], statusCode = _b[1], reasonPhrase = _b[2];
        return {
            headers: headers,
            body: body,
            statusCode: parseInt(statusCode, 10),
        };
    }
    throw new Error('Invalid response string');
}
exports.parseResponse = parseResponse;
/**
 * Formats HTTP requests into well-formed message strings.
 *
 * This function was written to aid in the creation of wire logs rather than to
 * be part of the critical path of an HTTP stack; it was not designed with
 * performance in mind.
 *
 * @see https://tools.ietf.org/html/rfc7230#section-3
 */
function serializeRequest(request) {
    var startLine = request.method + " " + request.path + " HTTP/1.1";
    return startLine + "\r\n" + serializeMessage(request);
}
exports.serializeRequest = serializeRequest;
/**
 * Formats HTTP responses into well-formed message strings.
 *
 * This function was written to aid in the creation of wire logs rather than to
 * be part of the critical path of an HTTP stack; it was not designed with
 * performance in mind.
 *
 * @see https://tools.ietf.org/html/rfc7230#section-3
 */
function serializeResponse(response) {
    var statusCode = response.statusCode;
    var startLine = "HTTP/1.1 " + statusCode + " " + constants_1.reasonPhrases[statusCode];
    return startLine + "\r\n" + serializeMessage(response);
}
exports.serializeResponse = serializeResponse;
function addHeaderToMessage(message, header) {
    var _a = tslib_1.__read(header, 2), name = _a[0], value = _a[1];
    if (message.headers[name]) {
        message.headers[name] += "," + value;
    }
    else {
        message.headers[name] = value;
    }
}
function parseAuthorityIntoMessage(message, authority) {
    var _a = tslib_1.__read(authority.split(':'), 2), hostname = _a[0], port = _a[1];
    message.hostname = hostname;
    if (port) {
        message.port = parseInt(port);
    }
}
function parseMessage(serialized) {
    var lines = serialized.split(/(?:\r)?\n/);
    var message = {
        startLine: lines.shift(),
        headers: {},
    };
    // parse as headers until an empty line is encountered
    var headerBeingParsed;
    for (var line = lines.shift(); line && line.trim(); line = lines.shift()) {
        if (line.indexOf(':') > -1) {
            // we've encountered a new header definition. Record the previous
            // header if one exists
            if (headerBeingParsed) {
                addHeaderToMessage(message, headerBeingParsed);
            }
            // begin parsing the new header
            var parts = line.split(':');
            headerBeingParsed = [
                parts[0].toLowerCase().trim(),
                parts.slice(1).join(':').trim()
            ];
        }
        else if (headerBeingParsed && line.match(/^\s+/)) {
            // this is a continuation of the previous header; remove a single
            // leading whitespace character and append to the previous header
            headerBeingParsed[1] += "," + line.trim();
        }
        else {
            throw new Error("Invalid line encountered: " + line);
        }
    }
    if (headerBeingParsed) {
        addHeaderToMessage(message, headerBeingParsed);
    }
    if (lines.length > 0) {
        message.body = lines.join('\n');
    }
    return message;
}
function parseQueryString(querystring) {
    return querystring.split('&').reduce(function (carry, headerPair) {
        var _a = tslib_1.__read(headerPair.split('=')), key = _a[0], valueParts = _a.slice(1);
        var value = valueParts.join('=');
        var prevValue = carry[key];
        if (prevValue) {
            if (Array.isArray(prevValue)) {
                prevValue.push(value);
            }
            else {
                carry[key] = [prevValue, value];
            }
        }
        else {
            carry[key] = value;
        }
        return carry;
    }, {});
}
/**
 * A regular expression that can decompose a well-formed URI into its
 * constituent parts
 *
 * @see https://tools.ietf.org/html/rfc3986#appendix-B
 */
var requestRegex = /^(([^:\/?#]+):)?(\/\/([^\/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?/;
/**
 * A target can take one of four forms: an absolute path, an absolute URL, an
 * asterisk (for OPTIONS requests), or an "authority" form (for CONNECT
 * requests) that includes the hostname and port, separated by a colon.
 *
 * @see https://tools.ietf.org/html/rfc7230#section-5.3
 */
function parseTargetIntoMessage(message, target) {
    if (target === '*') {
        // this is the asterisk form
        message.path = target;
    }
    else if (target.indexOf('/') === 0) {
        // this is the "origin form" (an absolute path)
        var _a = tslib_1.__read(target.split('?')), path = _a[0], additionalParts = _a.slice(1);
        message.path = path;
        if (additionalParts.length) {
            message.query = parseQueryString(additionalParts.join('?'));
        }
    }
    else if (target.indexOf('//') > -1) {
        // this is an absolute URL
        var matches = target.match(requestRegex);
        message.protocol = matches[1];
        parseAuthorityIntoMessage(message, matches[4]);
        message.path = matches[5];
        if (matches[7]) {
            message.query = parseQueryString(matches[7]);
        }
    }
    else {
        parseAuthorityIntoMessage(message, target);
    }
}
function serializeBody(message) {
    if (message.body === undefined) {
        return '';
    }
    if (typeof message.body === 'string') {
        return message.body;
    }
    if (ArrayBuffer.isView(message.body) || is_array_buffer_1.isArrayBuffer(message.body)) {
        return '[binary payload]';
    }
    return '[streaming payload]';
}
function serializeHeaders(message) {
    return Object.keys(message.headers).reduce(function (carry, key) { return carry + (key + ": " + message.headers[key] + "\r\n"); }, '');
}
function serializeMessage(message) {
    return serializeHeaders(message) + "\r\n" + serializeBody(message);
}
//# sourceMappingURL=index.js.map